@@ -27,7 +27,6 @@
 
 
 #include "pipe/p_compiler.h"
-#include "os/os_stream.h"
 #include "util/u_memory.h"
 #include "util/u_string.h"
 #include "util/u_format.h"
@@ -41,7 +40,7 @@
  */
 
 static INLINE void
-util_stream_writef(struct os_stream *stream, const char *format, ...)
+util_stream_writef(FILE *stream, const char *format, ...)
 {
    static char buf[1024];
    unsigned len;
@@ -49,102 +48,102 @@ util_stream_writef(struct os_stream *stream, const char *format, ...)
    va_start(ap, format);
    len = util_vsnprintf(buf, sizeof(buf), format, ap);
    va_end(ap);
-   os_stream_write(stream, buf, len);
+   fwrite(buf, len, 1, stream);
 }
 
 static void
-util_dump_bool(struct os_stream *stream, int value)
+util_dump_bool(FILE *stream, int value)
 {
    util_stream_writef(stream, "%c", value ? '1' : '0');
 }
 
 static void
-util_dump_int(struct os_stream *stream, long long int value)
+util_dump_int(FILE *stream, long long int value)
 {
    util_stream_writef(stream, "%lli", value);
 }
 
 static void
-util_dump_uint(struct os_stream *stream, long long unsigned value)
+util_dump_uint(FILE *stream, long long unsigned value)
 {
    util_stream_writef(stream, "%llu", value);
 }
 
 static void
-util_dump_float(struct os_stream *stream, double value)
+util_dump_float(FILE *stream, double value)
 {
    util_stream_writef(stream, "%g", value);
 }
 
 static void
-util_dump_string(struct os_stream *stream, const char *str)
+util_dump_string(FILE *stream, const char *str)
 {
-   os_stream_write_str(stream, "\"");
-   os_stream_write_str(stream, str);
-   os_stream_write_str(stream, "\"");
+   fputs("\"", stream);
+   fputs(str, stream);
+   fputs("\"", stream);
 }
 
 static void
-util_dump_enum(struct os_stream *stream, const char *value)
+util_dump_enum(FILE *stream, const char *value)
 {
-   os_stream_write_str(stream, value);
+   fputs(value, stream);
 }
 
 static void
-util_dump_array_begin(struct os_stream *stream)
+util_dump_array_begin(FILE *stream)
 {
-   os_stream_write_str(stream, "{");
+   fputs("{", stream);
 }
 
 static void
-util_dump_array_end(struct os_stream *stream)
+util_dump_array_end(FILE *stream)
 {
-   os_stream_write_str(stream, "}");
+   fputs("}", stream);
 }
 
 static void
-util_dump_elem_begin(struct os_stream *stream)
+util_dump_elem_begin(FILE *stream)
 {
 }
 
 static void
-util_dump_elem_end(struct os_stream *stream)
+util_dump_elem_end(FILE *stream)
 {
-   os_stream_write_str(stream, ", ");
+   fputs(", ", stream);
 }
 
 static void
-util_dump_struct_begin(struct os_stream *stream, const char *name)
+util_dump_struct_begin(FILE *stream, const char *name)
 {
-   os_stream_write_str(stream, "{");
+   fputs("{", stream);
 }
 
 static void
-util_dump_struct_end(struct os_stream *stream)
+util_dump_struct_end(FILE *stream)
 {
-   os_stream_write_str(stream, "}");
+   fputs("}", stream);
 }
 
 static void
-util_dump_member_begin(struct os_stream *stream, const char *name)
+util_dump_member_begin(FILE *stream, const char *name)
 {
    util_stream_writef(stream, "%s = ", name);
 }
 
 static void
-util_dump_member_end(struct os_stream *stream)
+util_dump_member_end(FILE *stream)
 {
-   os_stream_write_str(stream, ", ");
+   fputs(", ", stream);
 }
 
 static void
-util_dump_null(struct os_stream *stream)
+util_dump_null(FILE *stream)
 {
-   os_stream_write_str(stream, "NULL");
+   fputs("NULL", stream);
 }
 
 static void
-util_dump_ptr(struct os_stream *stream, const void *value)
+util_dump_ptr(FILE *stream, const void *value)
 {
    if(value)
       util_stream_writef(stream, "0x%08lx", (unsigned long)(uintptr_t)value);
@@ -224,26 +223,26 @@ util_dump_ptr(struct os_stream *stream, const void *value)
 
 
 static void
-util_dump_format(struct os_stream *stream, enum pipe_format format)
+util_dump_format(FILE *stream, enum pipe_format format)
 {
    util_dump_enum(stream, util_format_name(format));
 }
 
 
 static void
-util_dump_enum_blend_factor(struct os_stream *stream, unsigned value)
+util_dump_enum_blend_factor(FILE *stream, unsigned value)
 {
    util_dump_enum(stream, util_dump_blend_factor(value, TRUE));
 }
 
 static void
-util_dump_enum_blend_func(struct os_stream *stream, unsigned value)
+util_dump_enum_blend_func(FILE *stream, unsigned value)
 {
    util_dump_enum(stream, util_dump_blend_func(value, TRUE));
 }
 
 static void
-util_dump_enum_func(struct os_stream *stream, unsigned value)
+util_dump_enum_func(FILE *stream, unsigned value)
 {
    util_dump_enum(stream, util_dump_func(value, TRUE));
 }
@@ -255,14 +254,14 @@ util_dump_enum_func(struct os_stream *stream, unsigned value)
 
 
 void
-util_dump_template(struct os_stream *stream, const struct pipe_texture *templat)
+util_dump_template(FILE *stream, const struct pipe_resource *templat)
 {
    if(!templat) {
       util_dump_null(stream);
       return;
    }
 
-   util_dump_struct_begin(stream, "pipe_texture");
+   util_dump_struct_begin(stream, "pipe_resource");
 
    util_dump_member(stream, int, templat, target);
    util_dump_member(stream, format, templat, format);
@@ -279,15 +278,21 @@ util_dump_template(struct os_stream *stream, const struct pipe_texture *templat)
    util_dump_uint(stream, templat->depth0);
    util_dump_member_end(stream);
 
+   util_dump_member_begin(stream, "array_size");
+   util_dump_uint(stream, templat->array_size);
+   util_dump_member_end(stream);
+
    util_dump_member(stream, uint, templat, last_level);
-   util_dump_member(stream, uint, templat, tex_usage);
+   util_dump_member(stream, uint, templat, usage);
+   util_dump_member(stream, uint, templat, bind);
+   util_dump_member(stream, uint, templat, flags);
 
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_rasterizer_state(struct os_stream *stream, const struct pipe_rasterizer_state *state)
+util_dump_rasterizer_state(FILE *stream, const struct pipe_rasterizer_state *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -298,12 +303,15 @@ util_dump_rasterizer_state(struct os_stream *stream, const struct pipe_rasterize
 
    util_dump_member(stream, bool, state, flatshade);
    util_dump_member(stream, bool, state, light_twoside);
-   util_dump_member(stream, uint, state, front_winding);
-   util_dump_member(stream, uint, state, cull_mode);
-   util_dump_member(stream, uint, state, fill_cw);
-   util_dump_member(stream, uint, state, fill_ccw);
-   util_dump_member(stream, bool, state, offset_cw);
-   util_dump_member(stream, bool, state, offset_ccw);
+   util_dump_member(stream, bool, state, clamp_vertex_color);
+   util_dump_member(stream, bool, state, clamp_fragment_color);
+   util_dump_member(stream, uint, state, front_ccw);
+   util_dump_member(stream, uint, state, cull_face);
+   util_dump_member(stream, uint, state, fill_front);
+   util_dump_member(stream, uint, state, fill_back);
+   util_dump_member(stream, bool, state, offset_point);
+   util_dump_member(stream, bool, state, offset_line);
+   util_dump_member(stream, bool, state, offset_tri);
    util_dump_member(stream, bool, state, scissor);
    util_dump_member(stream, bool, state, poly_smooth);
    util_dump_member(stream, bool, state, poly_stipple_enable);
@@ -320,18 +328,22 @@ util_dump_rasterizer_state(struct os_stream *stream, const struct pipe_rasterize
    util_dump_member(stream, bool, state, line_last_pixel);
    util_dump_member(stream, bool, state, flatshade_first);
    util_dump_member(stream, bool, state, gl_rasterization_rules);
+   util_dump_member(stream, bool, state, rasterizer_discard);
+   util_dump_member(stream, bool, state, depth_clip);
+   util_dump_member(stream, uint, state, clip_plane_enable);
 
    util_dump_member(stream, float, state, line_width);
    util_dump_member(stream, float, state, point_size);
    util_dump_member(stream, float, state, offset_units);
    util_dump_member(stream, float, state, offset_scale);
+   util_dump_member(stream, float, state, offset_clamp);
 
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_poly_stipple(struct os_stream *stream, const struct pipe_poly_stipple *state)
+util_dump_poly_stipple(FILE *stream, const struct pipe_poly_stipple *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -349,7 +361,7 @@ util_dump_poly_stipple(struct os_stream *stream, const struct pipe_poly_stipple
 
 
 void
-util_dump_viewport_state(struct os_stream *stream, const struct pipe_viewport_state *state)
+util_dump_viewport_state(FILE *stream, const struct pipe_viewport_state *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -366,7 +378,7 @@ util_dump_viewport_state(struct os_stream *stream, const struct pipe_viewport_st
 
 
 void
-util_dump_scissor_state(struct os_stream *stream, const struct pipe_scissor_state *state)
+util_dump_scissor_state(FILE *stream, const struct pipe_scissor_state *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -385,7 +397,7 @@ util_dump_scissor_state(struct os_stream *stream, const struct pipe_scissor_stat
 
 
 void
-util_dump_clip_state(struct os_stream *stream, const struct pipe_clip_state *state)
+util_dump_clip_state(FILE *stream, const struct pipe_clip_state *state)
 {
    unsigned i;
 
@@ -406,16 +418,15 @@ util_dump_clip_state(struct os_stream *stream, const struct pipe_clip_state *sta
    util_dump_array_end(stream);
    util_dump_member_end(stream);
 
-   util_dump_member(stream, uint, state, nr);
-
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_shader_state(struct os_stream *stream, const struct pipe_shader_state *state)
+util_dump_shader_state(FILE *stream, const struct pipe_shader_state *state)
 {
    char str[8192];
+   unsigned i;
 
    if(!state) {
       util_dump_null(stream);
@@ -430,12 +441,30 @@ util_dump_shader_state(struct os_stream *stream, const struct pipe_shader_state
    util_dump_string(stream, str);
    util_dump_member_end(stream);
 
+   util_dump_member_begin(stream, "stream_output");
+   util_dump_struct_begin(stream, "pipe_stream_output_info");
+   util_dump_member(stream, uint, &state->stream_output, num_outputs);
+   util_dump_member(stream, uint, &state->stream_output, stride);
+   util_dump_array_begin(stream);
+   for(i = 0; i < state->stream_output.num_outputs; ++i) {
+      util_dump_elem_begin(stream);
+      util_dump_struct_begin(stream, ""); /* anonymous */
+      util_dump_member(stream, uint, &state->stream_output.output[i], register_index);
+      util_dump_member(stream, uint, &state->stream_output.output[i], register_mask);
+      util_dump_member(stream, uint, &state->stream_output.output[i], output_buffer);
+      util_dump_struct_end(stream);
+      util_dump_elem_end(stream);
+   }
+   util_dump_array_end(stream);
+   util_dump_struct_end(stream);
+   util_dump_member_end(stream);
+
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_depth_stencil_alpha_state(struct os_stream *stream, const struct pipe_depth_stencil_alpha_state *state)
+util_dump_depth_stencil_alpha_state(FILE *stream, const struct pipe_depth_stencil_alpha_state *state)
 {
    unsigned i;
 
@@ -490,7 +519,7 @@ util_dump_depth_stencil_alpha_state(struct os_stream *stream, const struct pipe_
 }
 
 void
-util_dump_rt_blend_state(struct os_stream *stream, const struct pipe_rt_blend_state *state)
+util_dump_rt_blend_state(FILE *stream, const struct pipe_rt_blend_state *state)
 {
    util_dump_struct_begin(stream, "pipe_rt_blend_state");
 
@@ -511,7 +540,7 @@ util_dump_rt_blend_state(struct os_stream *stream, const struct pipe_rt_blend_st
 }
 
 void
-util_dump_blend_state(struct os_stream *stream, const struct pipe_blend_state *state)
+util_dump_blend_state(FILE *stream, const struct pipe_blend_state *state)
 {
    unsigned valid_entries = 1;
 
@@ -543,7 +572,7 @@ util_dump_blend_state(struct os_stream *stream, const struct pipe_blend_state *s
 
 
 void
-util_dump_blend_color(struct os_stream *stream, const struct pipe_blend_color *state)
+util_dump_blend_color(FILE *stream, const struct pipe_blend_color *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -558,7 +587,7 @@ util_dump_blend_color(struct os_stream *stream, const struct pipe_blend_color *s
 }
 
 void
-util_dump_stencil_ref(struct os_stream *stream, const struct pipe_stencil_ref *state)
+util_dump_stencil_ref(FILE *stream, const struct pipe_stencil_ref *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -573,7 +602,7 @@ util_dump_stencil_ref(struct os_stream *stream, const struct pipe_stencil_ref *s
 }
 
 void
-util_dump_framebuffer_state(struct os_stream *stream, const struct pipe_framebuffer_state *state)
+util_dump_framebuffer_state(FILE *stream, const struct pipe_framebuffer_state *state)
 {
    util_dump_struct_begin(stream, "pipe_framebuffer_state");
 
@@ -588,7 +617,7 @@ util_dump_framebuffer_state(struct os_stream *stream, const struct pipe_framebuf
 
 
 void
-util_dump_sampler_state(struct os_stream *stream, const struct pipe_sampler_state *state)
+util_dump_sampler_state(FILE *stream, const struct pipe_sampler_state *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -610,14 +639,14 @@ util_dump_sampler_state(struct os_stream *stream, const struct pipe_sampler_stat
    util_dump_member(stream, float, state, lod_bias);
    util_dump_member(stream, float, state, min_lod);
    util_dump_member(stream, float, state, max_lod);
-   util_dump_member_array(stream, float, state, border_color);
+   util_dump_member_array(stream, float, state, border_color.f);
 
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_surface(struct os_stream *stream, const struct pipe_surface *state)
+util_dump_surface(FILE *stream, const struct pipe_surface *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -630,21 +659,19 @@ util_dump_surface(struct os_stream *stream, const struct pipe_surface *state)
    util_dump_member(stream, uint, state, width);
    util_dump_member(stream, uint, state, height);
 
-   util_dump_member(stream, uint, state, layout);
-   util_dump_member(stream, uint, state, offset);
    util_dump_member(stream, uint, state, usage);
 
    util_dump_member(stream, ptr, state, texture);
-   util_dump_member(stream, uint, state, face);
-   util_dump_member(stream, uint, state, level);
-   util_dump_member(stream, uint, state, zslice);
+   util_dump_member(stream, uint, state, u.tex.level);
+   util_dump_member(stream, uint, state, u.tex.first_layer);
+   util_dump_member(stream, uint, state, u.tex.last_layer);
 
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_transfer(struct os_stream *stream, const struct pipe_transfer *state)
+util_dump_transfer(FILE *stream, const struct pipe_transfer *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -653,23 +680,20 @@ util_dump_transfer(struct os_stream *stream, const struct pipe_transfer *state)
 
    util_dump_struct_begin(stream, "pipe_transfer");
 
-   util_dump_member(stream, uint, state, width);
-   util_dump_member(stream, uint, state, height);
+   util_dump_member(stream, ptr, state, resource);
+   /*util_dump_member(stream, uint, state, box);*/
 
    util_dump_member(stream, uint, state, stride);
-   util_dump_member(stream, uint, state, usage);
+   util_dump_member(stream, uint, state, layer_stride);
 
-   util_dump_member(stream, ptr, state, texture);
-   util_dump_member(stream, uint, state, face);
-   util_dump_member(stream, uint, state, level);
-   util_dump_member(stream, uint, state, zslice);
+   /*util_dump_member(stream, ptr, state, data);*/
 
    util_dump_struct_end(stream);
 }
 
 
 void
-util_dump_vertex_buffer(struct os_stream *stream, const struct pipe_vertex_buffer *state)
+util_dump_vertex_buffer(FILE *stream, const struct pipe_vertex_buffer *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -679,7 +703,6 @@ util_dump_vertex_buffer(struct os_stream *stream, const struct pipe_vertex_buffe
    util_dump_struct_begin(stream, "pipe_vertex_buffer");
 
    util_dump_member(stream, uint, state, stride);
-   util_dump_member(stream, uint, state, max_index);
    util_dump_member(stream, uint, state, buffer_offset);
    util_dump_member(stream, ptr, state, buffer);
 
@@ -688,7 +711,7 @@ util_dump_vertex_buffer(struct os_stream *stream, const struct pipe_vertex_buffe
 
 
 void
-util_dump_vertex_element(struct os_stream *stream, const struct pipe_vertex_element *state)
+util_dump_vertex_element(FILE *stream, const struct pipe_vertex_element *state)
 {
    if(!state) {
       util_dump_null(stream);
@@ -700,9 +723,40 @@ util_dump_vertex_element(struct os_stream *stream, const struct pipe_vertex_elem
    util_dump_member(stream, uint, state, src_offset);
 
    util_dump_member(stream, uint, state, vertex_buffer_index);
-   util_dump_member(stream, uint, state, nr_components);
 
    util_dump_member(stream, format, state, src_format);
 
    util_dump_struct_end(stream);
 }
+
+
+void
+util_dump_draw_info(FILE *stream, const struct pipe_draw_info *state)
+{
+   if(!state) {
+      util_dump_null(stream);
+      return;
+   }
+
+   util_dump_struct_begin(stream, "pipe_draw_info");
+
+   util_dump_member(stream, bool, state, indexed);
+
+   util_dump_member(stream, uint, state, mode);
+   util_dump_member(stream, uint, state, start);
+   util_dump_member(stream, uint, state, count);
+
+   util_dump_member(stream, uint, state, start_instance);
+   util_dump_member(stream, uint, state, instance_count);
+
+   util_dump_member(stream, int,  state, index_bias);
+   util_dump_member(stream, uint, state, min_index);
+   util_dump_member(stream, uint, state, max_index);
+
+   util_dump_member(stream, bool, state, primitive_restart);
+   util_dump_member(stream, uint, state, restart_index);
+
+   util_dump_member(stream, ptr, state, count_from_stream_output);
+
+   util_dump_struct_end(stream);
+}
