@@ -180,10 +180,10 @@ print_unpacked_z_32unorm(const struct util_format_description *format_desc,
 
 
 static void
-print_unpacked_s_8uscaled(const struct util_format_description *format_desc,
-                          const char *prefix,
-                          uint8_t unpacked[UTIL_FORMAT_MAX_UNPACKED_HEIGHT][UTIL_FORMAT_MAX_UNPACKED_WIDTH],
-                          const char *suffix)
+print_unpacked_s_8uint(const struct util_format_description *format_desc,
+                       const char *prefix,
+                       uint8_t unpacked[UTIL_FORMAT_MAX_UNPACKED_HEIGHT][UTIL_FORMAT_MAX_UNPACKED_WIDTH],
+                       const char *suffix)
 {
    unsigned i, j;
    const char *sep = "";
@@ -553,7 +553,7 @@ test_format_pack_z_32unorm(const struct util_format_description *format_desc,
 
 
 static boolean
-test_format_unpack_s_8uscaled(const struct util_format_description *format_desc,
+test_format_unpack_s_8uint(const struct util_format_description *format_desc,
                                const struct util_format_test_case *test)
 {
    uint8_t unpacked[UTIL_FORMAT_MAX_UNPACKED_HEIGHT][UTIL_FORMAT_MAX_UNPACKED_WIDTH] = { { 0 } };
@@ -561,7 +561,7 @@ test_format_unpack_s_8uscaled(const struct util_format_description *format_desc,
    unsigned i, j;
    boolean success;
 
-   format_desc->unpack_s_8uscaled(&unpacked[0][0], sizeof unpacked[0],
+   format_desc->unpack_s_8uint(&unpacked[0][0], sizeof unpacked[0],
                                   test->packed, 0,
                                   format_desc->block.width, format_desc->block.height);
 
@@ -581,8 +581,8 @@ test_format_unpack_s_8uscaled(const struct util_format_description *format_desc,
    }
 
    if (!success) {
-      print_unpacked_s_8uscaled(format_desc, "FAILED: ", unpacked, " obtained\n");
-      print_unpacked_s_8uscaled(format_desc, "        ", expected, " expected\n");
+      print_unpacked_s_8uint(format_desc, "FAILED: ", unpacked, " obtained\n");
+      print_unpacked_s_8uint(format_desc, "        ", expected, " expected\n");
    }
 
    return success;
@@ -590,7 +590,7 @@ test_format_unpack_s_8uscaled(const struct util_format_description *format_desc,
 
 
 static boolean
-test_format_pack_s_8uscaled(const struct util_format_description *format_desc,
+test_format_pack_s_8uint(const struct util_format_description *format_desc,
                              const struct util_format_test_case *test)
 {
    uint8_t unpacked[UTIL_FORMAT_MAX_UNPACKED_HEIGHT][UTIL_FORMAT_MAX_UNPACKED_WIDTH];
@@ -609,7 +609,7 @@ test_format_pack_s_8uscaled(const struct util_format_description *format_desc,
 
    memset(packed, 0, sizeof packed);
 
-   format_desc->pack_s_8uscaled(packed, 0,
+   format_desc->pack_s_8uint(packed, 0,
                                 &unpacked[0][0], sizeof unpacked[0],
                                 format_desc->block.width, format_desc->block.height);
 
@@ -638,7 +638,7 @@ test_one_func(const struct util_format_description *format_desc,
               const char *suffix)
 {
    unsigned i;
-   bool success = TRUE;
+   boolean success = TRUE;
 
    printf("Testing util_format_%s_%s ...\n",
           format_desc->short_name, suffix);
@@ -662,7 +662,7 @@ static boolean
 test_all(void)
 {
    enum pipe_format format;
-   bool success = TRUE;
+   boolean success = TRUE;
 
    for (format = 1; format < PIPE_FORMAT_COUNT; ++format) {
       const struct util_format_description *format_desc;
@@ -694,8 +694,8 @@ test_all(void)
       TEST_ONE_FUNC(pack_z_32unorm);
       TEST_ONE_FUNC(unpack_z_float);
       TEST_ONE_FUNC(pack_z_float);
-      TEST_ONE_FUNC(unpack_s_8uscaled);
-      TEST_ONE_FUNC(pack_s_8uscaled);
+      TEST_ONE_FUNC(unpack_s_8uint);
+      TEST_ONE_FUNC(pack_s_8uint);
 
 #     undef TEST_ONE_FUNC
    }
