@@ -29,8 +29,8 @@
 
 
 #include "main/glheader.h"
+#include "main/macros.h"
 #include "main/mtypes.h"		/* GLchan hack */
-#include "main/colormac.h"
 
 #include "m_translate.h"
 
