@@ -27,11 +27,14 @@
 #define S_TEXFILTER_H
 
 
-#include "swrast.h"
+#include "s_context.h"
+
+struct gl_context;
+struct gl_texture_object;
 
 
 extern texture_sample_func
-_swrast_choose_texture_sample_func( GLcontext *ctx,
+_swrast_choose_texture_sample_func( struct gl_context *ctx,
 				    const struct gl_texture_object *tObj );
 
 
